"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZRpcResult = exports.ZPayloadRpcData = exports.ZPayloadRpcArray = exports.ZPayloadRpc = exports.zVersion = void 0;
const zod_1 = require("zod");
const literalSchema = zod_1.z.union([zod_1.z.string(), zod_1.z.number(), zod_1.z.boolean(), zod_1.z.null()]);
const jsonSchema = zod_1.z.lazy(() => zod_1.z.union([literalSchema, zod_1.z.array(jsonSchema), zod_1.z.record(jsonSchema)]));
const zParams = zod_1.z.union([
    zod_1.z.string(),
    zod_1.z.number(),
    zod_1.z.boolean(),
    zod_1.z.null(),
    jsonSchema,
]);
exports.zVersion = zod_1.z.literal('2.0');
const zMethod = zod_1.z.string().transform((params, ctx) => {
    const result = params.split('.');
    if (result.length !== 2) {
        ctx.addIssue({
            code: zod_1.z.ZodIssueCode.custom,
            message: 'Not a 2 items',
        });
        return zod_1.z.NEVER;
    }
    return {
        spaceName: result[0],
        methodName: result[1],
    };
});
const zRpcParams = zod_1.z
    .union([zod_1.z.array(zParams), zod_1.z.record(zParams)])
    .transform((params) => Array.isArray(params) ? params : Object.values(params));
const zRpcId = zod_1.z.union([
    zod_1.z.string().regex(/^\d+$/).transform(Number),
    zod_1.z.number(),
]);
exports.ZPayloadRpc = zod_1.z.object({
    jsonrpc: exports.zVersion,
    method: zMethod,
    params: zRpcParams,
    id: zRpcId,
});
exports.ZPayloadRpcArray = zod_1.z.array(exports.ZPayloadRpc).min(1);
exports.ZPayloadRpcData = zod_1.z.union([exports.ZPayloadRpc, exports.ZPayloadRpcArray]);
exports.ZRpcResult = zod_1.z.object({
    jsonrpc: exports.zVersion,
    result: zRpcParams,
    id: zod_1.z.union([zRpcId, zod_1.z.null()]),
});
//# sourceMappingURL=payloade.js.map