"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcError = void 0;
exports.createErrorCustomError = createErrorCustomError;
exports.createError = createError;
exports.fromRpcErrorToRpcErrorObject = fromRpcErrorToRpcErrorObject;
exports.getBodyError = getBodyError;
const constants_1 = require("../constants");
const types_1 = require("../types");
class RpcError extends Error {
    constructor(message, code, data) {
        super(message);
        this.code = code;
        this.data = data;
        this.id = null;
    }
}
exports.RpcError = RpcError;
function getErrorData(title, description) {
    let data = undefined;
    if (title) {
        data = { title };
    }
    if (title && description) {
        data = { title, description };
    }
    return data;
}
function createErrorCustomError(code, title, description) {
    const absCode = Math.abs(code);
    let resultCode = 3200;
    if (code < 0 && absCode > 3200 && absCode <= 32099) {
        resultCode = code;
    }
    return new RpcError(types_1.ErrorCodeType.ServerError, resultCode, getErrorData(title, description));
}
function createError(type, title, description) {
    return new RpcError(type, constants_1.ErrorCode[type], getErrorData(title, description));
}
function fromRpcErrorToRpcErrorObject(error, id = null) {
    return {
        jsonrpc: '2.0',
        error: {
            message: error.message,
            code: error.code,
            ...(error.data ? { data: error.data } : {}),
        },
        id: error.id ? error.id : id,
    };
}
function getBodyError(exception) {
    if (exception instanceof RpcError) {
        return fromRpcErrorToRpcErrorObject(exception);
    }
    return fromRpcErrorToRpcErrorObject(createError(types_1.ErrorCodeType.ServerError, exception.message));
}
//# sourceMappingURL=error.js.map